/*
** MixerCurves.c
**
** Constructors and destructors for curves.
*/

#include"MixerProtos.h"
#include"DebugLib.h"
#include<stdlib.h>

t_point* add_point(point, x, y) /* Ajouter un point de vol, pan, etc... */
     t_point* point;
     int      x;
     int      y;
     
{
  t_point* temp_point;

  if (point == NULL) // liste vide
    {
      temp_point = (t_point*) malloc(sizeof(t_point));
      temp_point->x = x;
      temp_point->y = y;
      temp_point->next_point = NULL;
      return (temp_point);
    }

  if (point->next_point == NULL) // on est au bout de la liste
    {
     point->next_point = (t_point*) malloc(sizeof(t_point));
     point->next_point->x = x;
     point->next_point->y = y;
     point->next_point->next_point = NULL;
     return (point->next_point);
    }

  // On est au milieu
  temp_point = point->next_point;
  point->next_point = (t_point*) malloc(sizeof(t_point));
  point->next_point->x = x;
  point->next_point->y = y;
  point->next_point->next_point = temp_point;
  return (point->next_point);
   
} /* add_point */

int remove_point(first_point,point)
     t_point* first_point;
     t_point* point;
{ 
  t_point* tmppoint;
  
  if (point == NULL)
    {
      warning("MixerCurves: remove_point: I can't destroy a NULL point");
      return(-1);
    }

  tmppoint = first_point;

  if (tmppoint == NULL)
    {
      warning("MixerCurves: remove_point: I can't remove a point from a NULL list");
      return(-2);
    }

  if (tmppoint == point)
    {
      warning("MixerCurves: remove_point: I can't change the top pointer value of the list to NULL");
      free(point);
      return(1);
    }

  while((tmppoint->next_point != point) && (tmppoint->next_point != NULL)) 
    {
      tmppoint = tmppoint->next_point;
    }

  if (tmppoint->next_point == NULL)
    {
      warning("MixerCurves: remove_point: I can't find the point you want to remove in the list");
      return(-3);
    }
  
  tmppoint->next_point = point->next_point;
  free(point);
  return(0);

} // remove_point

int remove_vol_point(mixer_elt, x)
     t_mixer_elt* mixer_elt;
     int x;

{
  t_point* tmppoint;

  tmppoint = mixer_elt->mixer_info->vol_curve;
  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_vol_point: I can't remove a point from a"
	      " NULL volume curve");
      return(-1);
    }

  while((tmppoint != NULL) && (tmppoint->x != x))
    {
      tmppoint = tmppoint->next_point;
    }

  if (tmppoint == mixer_elt->mixer_info->vol_curve)
    {

      mixer_elt->mixer_info->vol_curve = tmppoint->next_point;
      free(tmppoint);
      return(0);
      
    }

  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_vol_point: I couldn't find the"
	      " point you asked me to remove");
      return(-1);
    }
  remove_point(mixer_elt->mixer_info->vol_curve, tmppoint);
  return(0);

}

int remove_pan_point(mixer_elt, x)
     t_mixer_elt* mixer_elt;
     int x;

{
  t_point* tmppoint;

  tmppoint = mixer_elt->mixer_info->pan_curve;
  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_pan_point: I can't remove a point from a"
	      " NULL panning curve");
      return(-1);
    }

  while((tmppoint != NULL) && (tmppoint->x != x))
    {
      tmppoint = tmppoint->next_point;
    }

  if (tmppoint == mixer_elt->mixer_info->pan_curve)
    {

      mixer_elt->mixer_info->pan_curve = tmppoint->next_point;
      free(tmppoint);
      return(0);
      
    }

  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_pan_point: I couldn't find the"
	      " point you asked me to remove");
      return(-1);
    }
  remove_point(mixer_elt->mixer_info->pan_curve, tmppoint);
  return(0);

}

int remove_frq_point(mixer_elt, x)
     t_mixer_elt* mixer_elt;
     int x;

{
  t_point* tmppoint;

  tmppoint = mixer_elt->mixer_info->frq_curve;
  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_frq_point: I can't remove a point from a"
	      " NULL frequency curve");
      return(-1);
    }

  while((tmppoint != NULL) && (tmppoint->x != x))
    {
      tmppoint = tmppoint->next_point;
    }

  if (tmppoint == mixer_elt->mixer_info->frq_curve)
    {

      mixer_elt->mixer_info->frq_curve = tmppoint->next_point;
      free(tmppoint);
      return(0);
      
    }

  if (tmppoint == NULL)
    {
      warning("MixerCurves.c: remove_frq_point: I couldn't find the"
	      " point you asked me to remove");
      return(-1);
    }
  remove_point(mixer_elt->mixer_info->frq_curve, tmppoint);
  return(0);

}

t_point* new_vol_curve(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  mixer_elt->mixer_info->vol_curve = add_point(mixer_elt->mixer_info->vol_curve, x, y);
  return(mixer_elt->mixer_info->vol_curve);
}

t_point* new_pan_curve(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  mixer_elt->mixer_info->pan_curve = add_point(mixer_elt->mixer_info->pan_curve, x, y);
  return(mixer_elt->mixer_info->pan_curve);
}

t_point* new_frq_curve(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  mixer_elt->mixer_info->frq_curve = add_point(mixer_elt->mixer_info->frq_curve, x, y);
  return(mixer_elt->mixer_info->frq_curve);
}

t_point* add_vol_point(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  t_point* point;
  t_point* tmppoint;
  t_point* result;

  point = mixer_elt->mixer_info->vol_curve;
  if (point == NULL)
    {
      result = new_vol_curve(mixer_elt,x,y);
    }
  else
    {
      if (point->x >= x) // first point of the curve
	{
	  tmppoint = point->next_point;
          result = new_vol_curve(mixer_elt,x,y);
	  result->next_point = point;
	  point->next_point = tmppoint;
	}
      else
	{
	  while ((point->next_point != NULL) && (point->next_point->x < x))
	    {
	      point = point->next_point;
	    }
	  
	  result=add_point(point,x,y);
	}
    }
  return(result);
  
}

t_point* add_pan_point(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  t_point* point;
  t_point* tmppoint;
  t_point* result;

  point = mixer_elt->mixer_info->pan_curve;
  if (point == NULL)
    {
      result = new_pan_curve(mixer_elt,x,y);
    }
  else
    {
      if (point->x >= x) // first point of the curve
	{
	  tmppoint = point->next_point;
          result = new_pan_curve(mixer_elt,x,y);
	  result->next_point = point;
	  point->next_point = tmppoint;
	}
      else
	{
	  while ((point->next_point != NULL) && (point->next_point->x < x))
	    {
	      point = point->next_point;
	    }
	  
	  result=add_point(point,x,y);
	}
    }
  return(result);
  
}

t_point* add_frq_point(mixer_elt, x, y)
     t_mixer_elt* mixer_elt;
     int          x;
     int          y;
     
{
  t_point* point;
  t_point* tmppoint;
  t_point* result;

  point = mixer_elt->mixer_info->frq_curve;
  if (point == NULL)
    {
      result = new_frq_curve(mixer_elt,x,y);
    }
  else
    {
      if (point->x >= x) // first point of the curve
	{
	  tmppoint = point->next_point;
          result = new_frq_curve(mixer_elt,x,y);
	  result->next_point = point;
	  point->next_point = tmppoint;
	}
      else
	{
	  while ((point->next_point != NULL) && (point->next_point->x < x))
	    {
	      point = point->next_point;
	    }
	  
	  result=add_point(point,x,y);
	}
    }
  return(result);
  
}
